
#include "windows.h"
#include "stdlib.h"
#include "helpline.h"

HANDLE hInst=0;
HWND hDlgOn=0;


int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
MSG msg;

	if (! hPrevInstance)
		if (! InitApplication (hInstance))
			return (FALSE);

	if (! InitInstance (hInstance, nCmdShow))
		return (FALSE);

	while (GetMessage (&msg, NULL, NULL, NULL))
		{
		if (hDlgOn)
			if (IsDialogMessage (hDlgOn, &msg))
				continue;
		TranslateMessage(&msg);
		DispatchMessage(&msg);
		}

	return (msg.wParam);
}

BOOL InitApplication (HANDLE hInstance)
{
WNDCLASS wc;

	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = MainWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 2;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon (hInstance, "HelpLine");
	wc.hCursor = LoadCursor (NULL, IDC_ARROW);
	wc.hbrBackground = GetStockObject(WHITE_BRUSH); 
	wc.lpszMenuName = "MainMenu";
	wc.lpszClassName = "GenericWClass";

	return (RegisterClass(&wc));
}

BOOL InitInstance (HANDLE hInstance, int nCmdShow)
{
HWND hWnd;

	hInst = hInstance;

	if (! (hWnd = CreateWindow("GenericWClass", "HelpLine Main Window",
					WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT,
					CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL)))
		return (FALSE);

	ShowWindow (hWnd, nCmdShow);
	UpdateWindow (hWnd);
	return (TRUE);
}

FARPROC lpProcDlg;

long FAR PASCAL MainWndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
UINT iLastPopUp;
FARPROC lpProcAbout;
PAINTSTRUCT ps;

	switch (message)
		{
		// Handle Menu Messages
		case WM_INITMENUPOPUP :
			if (! HIWORD (lParam))
				HelpOut (hWnd, iLastPopUp = LOWORD (lParam) + MNU_POP_ORIG);
			else 
				HelpOut (hWnd, iLastPopUp = MNU_POP_SYS);
			SetWindowWord (hWnd, 0, iLastPopUp);
			break;

		case WM_MENUSELECT :
			if ((HIWORD (lParam) != 0xFFFF) || (wParam != 0))
				{
				if (LOWORD (lParam) & MF_POPUP)
					HelpOut (hWnd, GetWindowWord (hWnd, 0));
				else 
					HelpOut (hWnd, wParam);
				break;
				}

			// Menu is going away
			SetWindowWord (hWnd, 0, 0);
			HelpOut (hWnd, 0);
			break;

		case WM_CREATE :
			lpProcDlg = MakeProcInstance (DlgProc, hInst);
			return (DefWindowProc (hWnd, message, wParam, lParam));

		case WM_COMMAND :
			switch (wParam)
				{
				case MNU_EXIT :
					DestroyWindow (hWnd);
					return (0);

				case MNU_DIALOG :
					CreateDialog (hInst, "HelpLineBox", hWnd, lpProcDlg);
					return (0);

				case MNU_ABOUT :
					lpProcAbout = MakeProcInstance (About, hInst);
					DialogBox (hInst, "AboutBox", hWnd, lpProcAbout);
					FreeProcInstance (lpProcAbout);
					return (0);
				}
			return (DefWindowProc (hWnd, message, wParam, lParam));

		case WM_PAINT :
			BeginPaint (hWnd, &ps);
			HelpOut (hWnd, GetWindowWord (hWnd, 0));
			EndPaint (hWnd, &ps);
			break;

		case WM_DESTROY :
			FreeProcInstance (lpProcDlg);
			PostQuitMessage(0);
			break;

		default :
			return (DefWindowProc (hWnd, message, wParam, lParam));
		}

	return (0);
}

void HelpOut (HWND hWnd, UINT idRes)
{
HDC hDc;
HBRUSH hBr;
char sHelp[82];
TEXTMETRIC tm;
RECT rect;

	// Get the help line
	if (idRes == 0)
		sHelp[0] = 0;
	else 
		if (! LoadString (hInst, idRes, sHelp, 80))
			return;

	if ((hDc = GetDC (hWnd)) != 0)
		{
		// Set the color
		SetBkMode (hDc, OPAQUE);
		SetBkColor (hDc, RGB (0, 0, 255));
		SetTextColor (hDc, RGB (255, 0, 0));

		// Get the window & font sizes
		GetTextMetrics (hDc, &tm);
		GetClientRect (hWnd, &rect);
		rect.top = rect.bottom - (tm.tmHeight+tm.tmExternalLeading);

		// Since sHelp can be blank - special case that
		if (sHelp[0])
			{
			// Write it
			TextOut (hDc, 0, rect.top, sHelp, strlen (sHelp));

			// Move fill rect left side over
			rect.left = LOWORD (GetTextExtent (hDc, sHelp, strlen (sHelp)));
			}

		// Blank the previous help text
		hBr = CreateSolidBrush (RGB (0, 0, 255));
		FillRect (hDc, &rect, hBr);
		DeleteObject (hBr);

		ReleaseDC (hWnd, hDc);
		}
}

// Bring us up in the middle of our parent
void DlgCenter (HWND hDlg)
{
HWND hWndPar;
int iX, iY;
RECT rcDlg, rcPar, rcDesk;

	if ((hWndPar = GetWindowWord (hDlg, GWW_HWNDPARENT)) == 0)
		return;

	GetWindowRect (GetDesktopWindow (), &rcDesk);
	GetWindowRect (hWndPar, &rcPar);
	GetWindowRect (hDlg, &rcDlg);

	// Center it - make sure its visible
	iX = __max (0, (rcPar.right + rcPar.left - rcDlg.right + rcDlg.left) /2);
	iY = __max (0, (rcPar.bottom + rcPar.top - rcDlg.bottom + rcDlg.top) /2);
	iX = __min (iX, (rcDesk.right - rcDesk.left) - (rcDlg.right - rcDlg.left));
	iY = __min (iY, (rcDesk.bottom - rcDesk.top) - (rcDlg.bottom - rcDlg.top));

	SetWindowPos (hDlg, 0, iX, iY, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}


BOOL FAR PASCAL DlgProc (HWND hDlg, unsigned message, WORD wParam, LONG lParam)
{
HWND hCtrl;
HWND hFocOn;
HWND hFocus;

	// See if we need to write the help line
	hFocus = (HWND) GetWindowLong (hDlg, DWL_USER);
	if ((hFocOn = GetFocus ()) != hFocus)
		if (GetParent (hFocOn) == hDlg)
			{
			SetWindowLong (hDlg, DWL_USER, hFocOn);
			HelpOut (hDlg, GetDlgCtrlID (hFocOn));
			}

	switch (message)
		{
		case WM_PAINT :
			HelpOut (hDlg, GetDlgCtrlID (hFocus));
			break;

		case WM_INITDIALOG :
			hDlgOn = hDlg;
			SetWindowLong (hDlg, DWL_USER, 0);
			DlgCenter (hDlg);

			// Init the dialog controls
			CheckRadioButton (hDlg, DLG_RADIO1, DLG_RADIO2, DLG_RADIO1);
			if ((hCtrl = GetDlgItem (hDlg, DLG_LIST)) != 0)
				{
				SendMessage (hCtrl, LB_ADDSTRING, 0, (LPARAM) (LPCSTR) "C++");
				SendMessage (hCtrl, LB_ADDSTRING, 0, (LPARAM) (LPCSTR) "C");
				SendMessage (hCtrl, LB_ADDSTRING, 0, (LPARAM) (LPCSTR) "Pascal");
				SendMessage (hCtrl, LB_ADDSTRING, 0, (LPARAM) (LPCSTR) "Basic");
				SendMessage (hCtrl, LB_ADDSTRING, 0, (LPARAM) (LPCSTR) "Fortran");
				}
			if ((hCtrl = GetDlgItem (hDlg, DLG_COMBO)) != 0)
				{
				SendMessage (hCtrl, CB_ADDSTRING, 0, (LPARAM) (LPCSTR) "Win/NT");
				SendMessage (hCtrl, CB_ADDSTRING, 0, (LPARAM) (LPCSTR) "Windows");
				SendMessage (hCtrl, CB_ADDSTRING, 0, (LPARAM) (LPCSTR) "DOS");
				SendMessage (hCtrl, CB_ADDSTRING, 0, (LPARAM) (LPCSTR) "OS/2");
				SendMessage (hCtrl, CB_ADDSTRING, 0, (LPARAM) (LPCSTR) "System 7");
				SendMessage (hCtrl, CB_ADDSTRING, 0, (LPARAM) (LPCSTR) "UNIX");
				}
			return (TRUE);

		// Exit when the user presses OK
		case WM_COMMAND :
			switch (wParam)
				{
				case IDOK :
					hDlgOn = 0;
					DestroyWindow (hDlg);
					return (TRUE);

				case DLG_RADIO1 :
				case DLG_RADIO2 :
					CheckRadioButton (hDlg, DLG_RADIO1, DLG_RADIO2, wParam);
					return (TRUE);
				}
			break;

		case WM_ACTIVATE :
			if (wParam == WA_INACTIVE)
				hDlgOn = 0;
			else
				hDlgOn = hDlg;
			break;
		}

	return (FALSE);
}


BOOL FAR PASCAL About (HWND hDlg, unsigned message, WORD wParam, LONG lParam)
{

	switch (message)
		{
		case WM_INITDIALOG :
			// Set the Window help line to where we are
			HelpOut (GetWindowWord (hDlg, GWW_HWNDPARENT), STR_IN_ABOUT);

			DlgCenter (hDlg);
			return (TRUE);

		// Exit when the user presses OK
		case WM_COMMAND :
			if (wParam == IDOK)
				{
				HelpOut (GetWindowWord (hDlg, GWW_HWNDPARENT), 0);
				EndDialog (hDlg, TRUE);
				return (TRUE);
				}
			break;
		}

	return (FALSE);
}
